@@ -1,4 +1,4 @@
-from django.db.models import get_model
+from oscar.core.loading import get_model
 from django.utils.translation import ugettext_lazy as _
 from oscar.core.loading import get_class
 
@@ -55,6 +55,7 @@ class OpenBasketReportGenerator(ReportGenerator):
     """
     code = 'open_baskets'
     description = _('Open baskets')
+    date_range_field_name = 'date_created'
 
     formatters = {
         'CSV_formatter': OpenBasketReportCSVFormatter,
@@ -109,6 +110,7 @@ class SubmittedBasketReportGenerator(ReportGenerator):
     """
     code = 'submitted_baskets'
     description = _('Submitted baskets')
+    date_range_field_name = 'date_submitted'
 
     formatters = {
         'CSV_formatter': SubmittedBasketReportCSVFormatter,
