@@ -4,10 +4,10 @@ from django.core import mail
 from django.conf import settings
 from django.template import loader, Context
 from django.contrib.sites.models import Site
-from django.db.models import get_model, Max
+from django.db.models import Max
 
 from oscar.apps.customer.notifications import services
-from oscar.core.loading import get_class
+from oscar.core.loading import get_class, get_model
 
 ProductAlert = get_model('customer', 'ProductAlert')
 Product = get_model('catalogue', 'Product')
@@ -69,10 +69,12 @@ def send_product_alerts(product):
     num_alerts = alerts.count()
     if num_stockrecords == 1:
         num_in_stock = stockrecords[0].num_in_stock
-        hurry_mode = num_alerts < num_in_stock
+        # hurry_mode is false if num_in_stock is None
+        hurry_mode = num_in_stock is not None and num_alerts < num_in_stock
     else:
         result = stockrecords.aggregate(max_in_stock=Max('num_in_stock'))
-        hurry_mode = num_alerts < result['max_in_stock']
+        hurry_mode = result['max_in_stock'] is not None and \
+            num_alerts < result['max_in_stock']
 
     # Load templates
     message_tpl = loader.get_template('customer/alerts/message.html')
