@@ -30,6 +30,7 @@ class CustomerApplication(Application):
     register_view = views.AccountRegistrationView
     profile_view = views.ProfileView
     profile_update_view = views.ProfileUpdateView
+    profile_delete_view = views.ProfileDeleteView
     change_password_view = views.ChangePasswordView
 
     notification_inbox_view = notification_views.InboxView
@@ -55,12 +56,14 @@ class CustomerApplication(Application):
 
     def get_urls(self):
         urls = [
-            url(r'^$', login_required(self.summary_view.as_view()),
-                name='summary'),
+            # Login, logout and register doesn't require login
             url(r'^login/$', self.login_view.as_view(), name='login'),
             url(r'^logout/$', self.logout_view.as_view(), name='logout'),
             url(r'^register/$', self.register_view.as_view(), name='register'),
-            url(r'^change-password/$', self.change_password_view.as_view(),
+            url(r'^$', login_required(self.summary_view.as_view()),
+                name='summary'),
+            url(r'^change-password/$',
+                login_required(self.change_password_view.as_view()),
                 name='change-password'),
 
             # Profile
@@ -70,6 +73,9 @@ class CustomerApplication(Application):
             url(r'^profile/edit/$',
                 login_required(self.profile_update_view.as_view()),
                 name='profile-update'),
+            url(r'^profile/delete/$',
+                login_required(self.profile_delete_view.as_view()),
+                name='profile-delete'),
 
             # Order history
             url(r'^orders/$',
@@ -111,6 +117,7 @@ class CustomerApplication(Application):
                 name='email-detail'),
 
             # Notifications
+            # Redirect to notification inbox
             url(r'^notifications/$', generic.RedirectView.as_view(
                 url='/accounts/notifications/inbox/')),
             url(r'^notifications/inbox/$',
@@ -127,6 +134,8 @@ class CustomerApplication(Application):
                 name='notifications-detail'),
 
             # Alerts
+            # Alerts can be setup by anonymous users: some views do not
+            # require login
             url(r'^alerts/$',
                 login_required(self.alert_list_view.as_view()),
                 name='alerts-list'),
@@ -159,6 +168,7 @@ class CustomerApplication(Application):
             url(r'wishlists/create/with-product/(?P<product_pk>\d+)/$',
                 login_required(self.wishlists_create_view.as_view()),
                 name='wishlists-create-with-product'),
+            # Wishlists can be publicly shared, no login required
             url(r'wishlists/(?P<key>[a-z0-9]+)/$',
                 self.wishlists_detail_view.as_view(), name='wishlists-detail'),
             url(r'wishlists/(?P<key>[a-z0-9]+)/update/$',
@@ -179,6 +189,7 @@ class CustomerApplication(Application):
                 login_required(self.wishlists_move_product_to_another_view
                                .as_view()),
                 name='wishlists-move-product-to-another')]
+
         return self.post_process_urls(patterns('', *urls))
 
 
