@@ -1,22 +1,20 @@
 import string
-import urlparse
 import random
 
 from django import forms
 from django.conf import settings
 from django.contrib.auth import forms as auth_forms
 from django.contrib.auth.forms import AuthenticationForm
-from django.contrib.auth.tokens import default_token_generator
 from django.contrib.sites.models import get_current_site
-from django.core import validators
-from django.core.exceptions import ObjectDoesNotExist
 from django.core.exceptions import ValidationError
-from django.db.models import get_model
 from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import pgettext_lazy
 
-from oscar.core.loading import get_profile_class, get_class
-from oscar.core.compat import get_user_model
+from oscar.core.loading import get_profile_class, get_class, get_model
+from oscar.core.compat import get_user_model, existing_user_fields, urlparse
 from oscar.apps.customer.utils import get_password_reset_url, normalise_email
+from oscar.core.validators import password_validators
+
 
 Dispatcher = get_class('customer.utils', 'Dispatcher')
 CommunicationEventType = get_model('customer', 'communicationeventtype')
@@ -25,7 +23,12 @@ User = get_user_model()
 
 
 def generate_username():
-    uname = ''.join([random.choice(string.letters + string.digits + '_')
+    # Python 3 uses ascii_letters. If not available, fallback to letters
+    try:
+        letters = string.ascii_letters
+    except AttributeError:
+        letters = string.letters
+    uname = ''.join([random.choice(letters + string.digits + '_')
                      for i in range(30)])
     try:
         User.objects.get(username=uname)
@@ -40,11 +43,8 @@ class PasswordResetForm(auth_forms.PasswordResetForm):
     """
     communication_type_code = "PASSWORD_RESET"
 
-    def save(self, domain_override=None,
-             subject_template_name='registration/password_reset_subject.txt',
-             email_template_name='registration/password_reset_email.html',
-             use_https=False, token_generator=default_token_generator,
-             from_email=None, request=None, **kwargs):
+    def save(self, domain_override=None, use_https=False, request=None,
+             **kwargs):
         """
         Generates a one-use only link for resetting password and sends to the
         user.
@@ -69,6 +69,20 @@ class PasswordResetForm(auth_forms.PasswordResetForm):
             Dispatcher().dispatch_user_messages(user, messages)
 
 
+class SetPasswordForm(auth_forms.SetPasswordForm):
+    def __init__(self, *args, **kwargs):
+        super(SetPasswordForm, self).__init__(*args, **kwargs)
+        # Enforce password validations for the new password
+        self.fields['new_password1'].validators += password_validators
+
+
+class PasswordChangeForm(auth_forms.PasswordChangeForm):
+    def __init__(self, *args, **kwargs):
+        super(PasswordChangeForm, self).__init__(*args, **kwargs)
+        # Enforce password validations for the new password
+        self.fields['new_password1'].validators += password_validators
+
+
 class EmailAuthenticationForm(AuthenticationForm):
     """
     Extends the standard django AuthenticationForm, to support 75 character
@@ -93,68 +107,31 @@ class EmailAuthenticationForm(AuthenticationForm):
         return url
 
 
-class CommonPasswordValidator(validators.BaseValidator):
-    # See
-    # http://www.smartplanet.com/blog/business-brains/top-20-most-common-passwords-of-all-time-revealed-8216123456-8216princess-8216qwerty/4519  # noqa
-    forbidden_passwords = [
-        'password',
-        '1234',
-        '12345'
-        '123456',
-        '123456y',
-        '123456789',
-        'iloveyou',
-        'princess',
-        'monkey',
-        'rockyou',
-        'babygirl',
-        'monkey',
-        'qwerty',
-        '654321',
-        'dragon',
-        'pussy',
-        'baseball',
-        'football',
-        'letmein',
-        'monkey',
-        '696969',
-        'abc123',
-        'qwe123',
-        'qweasd',
-        'mustang',
-        'michael',
-        'shadow',
-        'master',
-        'jennifer',
-        '111111',
-        '2000',
-        'jordan',
-        'superman'
-        'harley'
-    ]
-    message = _("Please choose a less common password")
-    code = 'password'
+class ConfirmPasswordForm(forms.Form):
+    """
+    Extends the standard django AuthenticationForm, to support 75 character
+    usernames. 75 character usernames are needed to support the EmailOrUsername
+    auth backend.
+    """
+    password = forms.CharField(label=_("Password"), widget=forms.PasswordInput)
 
-    def __init__(self, password_file=None):
-        self.limit_value = password_file
+    def __init__(self, user, *args, **kwargs):
+        super(ConfirmPasswordForm, self).__init__(*args, **kwargs)
+        self.user = user
 
-    def clean(self, value):
-        return value.strip()
-
-    def compare(self, value, limit):
-        return value in self.forbidden_passwords
-
-    def get_forbidden_passwords(self):
-        if self.limit_value is None:
-            return self.forbidden_passwords
+    def clean_password(self):
+        password = self.cleaned_data['password']
+        if not self.user.check_password(password):
+            raise forms.ValidationError(
+                _("The entered password is not valid!"))
+        return password
 
 
 class EmailUserCreationForm(forms.ModelForm):
     email = forms.EmailField(label=_('Email address'))
     password1 = forms.CharField(
         label=_('Password'), widget=forms.PasswordInput,
-        validators=[validators.MinLengthValidator(6),
-                    CommonPasswordValidator()])
+        validators=password_validators)
     password2 = forms.CharField(
         label=_('Confirm password'), widget=forms.PasswordInput)
     redirect_url = forms.CharField(
@@ -172,7 +149,7 @@ class EmailUserCreationForm(forms.ModelForm):
         email = normalise_email(self.cleaned_data['email'])
         if User._default_manager.filter(email=email).exists():
             raise forms.ValidationError(
-                _("A user with that email address already exists."))
+                _("A user with that email address already exists"))
         return email
 
     def clean_password2(self):
@@ -204,8 +181,10 @@ class EmailUserCreationForm(forms.ModelForm):
 
 
 class OrderSearchForm(forms.Form):
-    date_from = forms.DateField(required=False, label=_("From"))
-    date_to = forms.DateField(required=False, label=_("To"))
+    date_from = forms.DateField(
+        required=False, label=pgettext_lazy("start date", "From"))
+    date_to = forms.DateField(
+        required=False, label=pgettext_lazy("end date", "To"))
     order_number = forms.CharField(required=False, label=_("Order number"))
 
     def clean(self):
@@ -301,35 +280,28 @@ class UserForm(forms.ModelForm):
 
     class Meta:
         model = User
-        exclude = ('username', 'password', 'is_staff', 'is_superuser',
-                   'is_active', 'last_login', 'date_joined',
-                   'user_permissions', 'groups')
+        fields = existing_user_fields(['first_name', 'last_name', 'email'])
 
 
 Profile = get_profile_class()
 if Profile:
 
     class UserAndProfileForm(forms.ModelForm):
-        email = forms.EmailField(label=_('Email address'), required=True)
 
         def __init__(self, user, *args, **kwargs):
-            self.user = user
             try:
                 instance = Profile.objects.get(user=user)
-            except ObjectDoesNotExist:
+            except Profile.DoesNotExist:
                 # User has no profile, try a blank one
                 instance = Profile(user=user)
             kwargs['instance'] = instance
 
             super(UserAndProfileForm, self).__init__(*args, **kwargs)
 
-            # Get a list of profile fields to help with ordering later
+            # Get profile field names to help with ordering later
             profile_field_names = self.fields.keys()
-            del profile_field_names[profile_field_names.index('email')]
 
-            self.fields['email'].initial = self.instance.user.email
-
-            # Add user fields (we look for core user fields first)
+            # Get user field names (we look for core user fields first)
             core_field_names = set([f.name for f in User._meta.fields])
             user_field_names = ['email']
             for field_name in ('first_name', 'last_name'):
@@ -340,11 +312,14 @@ if Profile:
             # Store user fields so we know what to save later
             self.user_field_names = user_field_names
 
-            # Add additional user fields
+            # Add additional user form fields
             additional_fields = forms.fields_for_model(
                 User, fields=user_field_names)
             self.fields.update(additional_fields)
 
+            # Ensure email is required and initialised correctly
+            self.fields['email'].required = True
+
             # Set initial values
             for field_name in user_field_names:
                 self.fields[field_name].initial = getattr(user, field_name)
@@ -359,7 +334,7 @@ if Profile:
         def clean_email(self):
             email = normalise_email(self.cleaned_data['email'])
             if User._default_manager.filter(
-                    email=email).exclude(id=self.user.id).exists():
+                    email=email).exclude(id=self.instance.user.id).exists():
                 raise ValidationError(
                     _("A user with this email address already exists"))
             return email
