@@ -17,5 +17,5 @@ class ProductAlert(abstract_models.AbstractProductAlert):
     pass
 
 
-from oscar.apps.customer.history_helpers import *
-from oscar.apps.customer.alerts.receivers import *
\ No newline at end of file
+from oscar.apps.customer.history import *  # noqa
+from oscar.apps.customer.alerts.receivers import *  # noqa
