@@ -14,7 +14,7 @@ class CommsDashboardApplication(Application):
     def get_urls(self):
         urls = [
             url(r'^$', self.list_view.as_view(), name='comms-list'),
-            url(r'^(?P<code>\w+)/$', self.update_view.as_view(),
+            url(r'^(?P<slug>\w+)/$', self.update_view.as_view(),
                 name='comms-update'),
         ]
         return self.post_process_urls(patterns('', *urls))
