@@ -1,7 +1,6 @@
 from django.contrib import messages
 from django.contrib.sites.models import get_current_site
-from django.db.models import get_model
-from django.shortcuts import get_object_or_404
+from oscar.core.loading import get_model
 from django.template import TemplateSyntaxError
 from django.utils.translation import ugettext_lazy as _
 from django.views import generic
@@ -26,10 +25,7 @@ class UpdateView(generic.UpdateView):
     template_name = 'dashboard/comms/detail.html'
     context_object_name = 'commtype'
     success_url = '.'
-
-    def get_object(self, **kwargs):
-        return get_object_or_404(self.model,
-                                 code=self.kwargs['code'].upper())
+    slug_field = 'code'
 
     def form_invalid(self, form):
         messages.error(self.request,
@@ -59,7 +55,7 @@ class UpdateView(generic.UpdateView):
         commtype_ctx = self.get_messages_context(form)
         try:
             msgs = commtype.get_messages(commtype_ctx)
-        except TemplateSyntaxError, e:
+        except TemplateSyntaxError as e:
             form.errors['__all__'] = form.error_class([e.message])
             return self.render_to_response(ctx)
 
@@ -75,7 +71,7 @@ class UpdateView(generic.UpdateView):
         commtype_ctx = self.get_messages_context(form)
         try:
             msgs = commtype.get_messages(commtype_ctx)
-        except TemplateSyntaxError, e:
+        except TemplateSyntaxError as e:
             form.errors['__all__'] = form.error_class([e.message])
             return self.render_to_response(ctx)
 
