@@ -1,3 +1,4 @@
+import six
 import datetime
 from decimal import Decimal as D, InvalidOperation
 
@@ -5,7 +6,7 @@ from django.contrib import messages
 from django.utils.translation import ugettext_lazy as _
 from django.core.urlresolvers import reverse
 from django.core.exceptions import ObjectDoesNotExist
-from django.db.models.loading import get_model
+from oscar.core.loading import get_model
 from django.db.models import fields, Q, Sum, Count
 from django.http import HttpResponse, HttpResponseRedirect, Http404
 from django.shortcuts import get_object_or_404
@@ -20,7 +21,8 @@ from oscar.views import sort_queryset
 from oscar.views.generic import BulkEditMixin
 from oscar.apps.dashboard.reports.csv_utils import CsvUnicodeWriter
 from oscar.apps.payment.exceptions import PaymentError
-from oscar.apps.order.exceptions import InvalidShippingEvent, InvalidStatus
+from oscar.apps.order.exceptions import (
+    InvalidShippingEvent, InvalidStatus, InvalidPaymentEvent)
 
 Order = get_model('order', 'Order')
 OrderNote = get_model('order', 'OrderNote')
@@ -40,7 +42,11 @@ def queryset_orders_for_user(user):
     To allow access to an order for a non-staff user, at least one line's
     partner has to have the user in the partner's list.
     """
-    queryset = Order._default_manager.all()
+    queryset = Order._default_manager.select_related(
+        'billing_address', 'billing_address__country',
+        'shipping_address', 'shipping_address__country',
+        'user'
+        ).prefetch_related('lines')
     if user.is_staff:
         return queryset
     else:
@@ -351,7 +357,7 @@ class OrderListView(BulkEditMixin, ListView):
             else:
                 row['billing_address_name'] = ''
 
-            encoded_values = [unicode(value).encode('utf8')
+            encoded_values = [six.text_type(value).encode('utf8')
                               for value in row.values()]
             writer.writerow(encoded_values)
         return response
@@ -360,6 +366,7 @@ class OrderListView(BulkEditMixin, ListView):
 class OrderDetailView(DetailView):
     """
     Dashboard view to display a single order.
+
     Supports the permission-based dashboard.
     """
     model = Order
@@ -417,21 +424,50 @@ class OrderDetailView(DetailView):
         line_action = request.POST.get('line_action', '').lower()
         if line_action:
             if line_action not in self.line_actions:
-                messages.error(self.request, "Invalid action")
+                messages.error(self.request, _("Invalid action"))
                 return self.reload_page_response()
-            else:
-                line_ids = request.POST.getlist('selected_line')
-                line_quantities = []
-                for line_id in line_ids:
-                    qty = request.POST.get('selected_line_qty_%s' % line_id)
-                    line_quantities.append(int(qty))
-                lines = order.lines.filter(id__in=line_ids)
-                if not lines.exists():
-                    messages.error(self.request,
-                                   _("You must select some lines to act on"))
+
+            # Load requested lines
+            line_ids = request.POST.getlist('selected_line')
+            if len(line_ids) == 0:
+                messages.error(
+                    self.request, _("You must select some lines to act on"))
+                return self.reload_page_response()
+
+            lines = order.lines.filter(id__in=line_ids)
+            if len(line_ids) != len(lines):
+                messages.error(
+                    self.request, _("Invalid lines requested"))
+                return self.reload_page_response()
+
+            # Build list of line quantities
+            line_quantities = []
+            for line in lines:
+                qty = request.POST.get('selected_line_qty_%s' % line.id)
+                try:
+                    qty = int(qty)
+                except ValueError:
+                    messages.error(
+                        self.request, _("The entered quantity for line #%s is not valid") % line.id)
                     return self.reload_page_response()
-                return getattr(self, line_action)(request, order, lines,
-                                                  line_quantities)
+                if qty <= 0:
+                    messages.error(
+                        self.request, _(
+                            "The entered quantity for line #%s is not valid") % line.id)
+                    return self.reload_page_response()
+                elif qty > line.quantity:
+                    messages.error(
+                        self.request, _(
+                            "The entered quantity for line #(line_id)%s "
+                            "should not be higher than %(quantity)s") % {
+                                'line_id': line.id,
+                                'quantity': line.quantity})
+                    return self.reload_page_response()
+
+                line_quantities.append(int(qty))
+
+            return getattr(self, line_action)(
+                request, order, lines, line_quantities)
 
         messages.error(request, _("No valid action submitted"))
         return self.reload_page_response()
@@ -478,14 +514,15 @@ class OrderDetailView(DetailView):
             return self.reload_page_response()
 
         handler = EventHandler(request.user)
+        old_status = order.status
         try:
             handler.handle_order_status_change(order, new_status)
-        except PaymentError, e:
+        except PaymentError as e:
             messages.error(request, _("Unable to change order status due to"
                                       " payment error: %s") % e)
         else:
             msg = _("Order status changed from '%(old_status)s' to"
-                    " '%(new_status)s'") % {'old_status': order.status,
+                    " '%(new_status)s'") % {'old_status': old_status,
                                             'new_status': new_status}
             messages.info(request, msg)
             order.notes.create(user=request.user, message=msg,
@@ -536,13 +573,13 @@ class OrderDetailView(DetailView):
             EventHandler().handle_shipping_event(order, event_type, lines,
                                                  quantities,
                                                  reference=reference)
-        except InvalidShippingEvent, e:
+        except InvalidShippingEvent as e:
             messages.error(request,
                            _("Unable to create shipping event: %s") % e)
-        except InvalidStatus, e:
+        except InvalidStatus as e:
             messages.error(request,
                            _("Unable to create shipping event: %s") % e)
-        except PaymentError, e:
+        except PaymentError as e:
             messages.error(request, _("Unable to create shipping event due to"
                                       " payment error: %s") % e)
         else:
@@ -550,6 +587,9 @@ class OrderDetailView(DetailView):
         return self.reload_page_response()
 
     def create_order_payment_event(self, request, order):
+        """
+        Create a payment event for the whole order
+        """
         amount_str = request.POST.get('amount', None)
         try:
             amount = D(amount_str)
@@ -558,26 +598,10 @@ class OrderDetailView(DetailView):
             return self.reload_page_response()
         return self._create_payment_event(request, order, amount)
 
-    def _create_payment_event(self, request, order, amount, lines=None,
-                              quantities=None):
-        code = request.POST['payment_event_type']
-        try:
-            event_type = PaymentEventType._default_manager.get(code=code)
-        except PaymentEventType.DoesNotExist:
-            messages.error(request, _("The event type '%s' is not valid")
-                           % code)
-            return self.reload_page_response()
-        try:
-            EventHandler().handle_payment_event(order, event_type, amount,
-                                                lines, quantities)
-        except PaymentError, e:
-            messages.error(request, _("Unable to change order status due to"
-                                      " payment error: %s") % e)
-        else:
-            messages.info(request, _("Payment event created"))
-        return self.reload_page_response()
-
     def create_payment_event(self, request, order, lines, quantities):
+        """
+        Create a payment event for a subset of order lines
+        """
         amount_str = request.POST.get('amount', None)
 
         # If no amount passed, then we add up the total of the selected lines
@@ -595,6 +619,28 @@ class OrderDetailView(DetailView):
         return self._create_payment_event(request, order, amount, lines,
                                           quantities)
 
+    def _create_payment_event(self, request, order, amount, lines=None,
+                              quantities=None):
+        code = request.POST.get('payment_event_type')
+        try:
+            event_type = PaymentEventType._default_manager.get(code=code)
+        except PaymentEventType.DoesNotExist:
+            messages.error(
+                request, _("The event type '%s' is not valid") % code)
+            return self.reload_page_response()
+        try:
+            EventHandler().handle_payment_event(
+                order, event_type, amount, lines, quantities)
+        except PaymentError as e:
+            messages.error(request, _("Unable to create payment event due to"
+                                      " payment error: %s") % e)
+        except InvalidPaymentEvent as e:
+            messages.error(
+                request, _("Unable to create payment event: %s") % e)
+        else:
+            messages.info(request, _("Payment event created"))
+        return self.reload_page_response()
+
 
 class LineDetailView(DetailView):
     """
