@@ -1,5 +1,5 @@
 from django import forms
-from django.db.models.loading import get_model
+from oscar.core.loading import get_model
 from django.utils.translation import ugettext_lazy as _
 
 from oscar.core.validators import URLDoesNotExistValidator
