@@ -1,7 +1,9 @@
+import six
+import operator
 from decimal import Decimal as D, ROUND_DOWN, ROUND_UP
 
 from django.core import exceptions
-from django.db.models import get_model
+from oscar.core.loading import get_model
 from django.template.defaultfilters import date as date_filter
 from django.db import models
 from django.utils.timezone import now, get_current_timezone
@@ -12,10 +14,10 @@ from django.core.urlresolvers import reverse
 from django.conf import settings
 
 from oscar.core.utils import slugify
+from oscar.core.loading import get_class
 from oscar.apps.offer.managers import ActiveOfferManager
 from oscar.templatetags.currency_filters import currency
-from oscar.models.fields import PositiveDecimalField, ExtendedURLField
-from oscar.core.loading import get_class
+from oscar.models import fields
 
 BrowsableRangeManager = get_class('offer.managers', 'BrowsableRangeManager')
 
@@ -24,7 +26,7 @@ def load_proxy(proxy_class):
     module, classname = proxy_class.rsplit('.', 1)
     try:
         mod = import_module(module)
-    except ImportError, e:
+    except ImportError as e:
         raise exceptions.ImproperlyConfigured(
             "Error importing module %s: %s" % (module, e))
     try:
@@ -63,7 +65,8 @@ class ConditionalOffer(models.Model):
     name = models.CharField(
         _("Name"), max_length=128, unique=True,
         help_text=_("This is displayed within the customer's basket"))
-    slug = models.SlugField(_("Slug"), max_length=128, unique=True, null=True)
+    slug = fields.AutoSlugField(
+        _("Slug"), max_length=128, unique=True, populate_from='name')
     description = models.TextField(_("Description"), blank=True,
                                    help_text=_("This is displayed on the offer"
                                                " browsing page"))
@@ -158,7 +161,8 @@ class ConditionalOffer(models.Model):
     num_orders = models.PositiveIntegerField(
         _("Number of Orders"), default=0)
 
-    redirect_url = ExtendedURLField(_("URL redirect (optional)"), blank=True)
+    redirect_url = fields.ExtendedURLField(
+        _("URL redirect (optional)"), blank=True)
     date_created = models.DateTimeField(_("Date Created"), auto_now_add=True)
 
     objects = models.Manager()
@@ -173,14 +177,7 @@ class ConditionalOffer(models.Model):
         verbose_name = _("Conditional offer")
         verbose_name_plural = _("Conditional offers")
 
-        # The way offers are looked up involves the fields (offer_type, status,
-        # start_datetime, end_datetime).  Ideally, you want a DB index that
-        # covers these 4 fields (will add support for this in Django 1.5)
-
     def save(self, *args, **kwargs):
-        if not self.slug:
-            self.slug = slugify(self.name)
-
         # Check to see if consumption thresholds have been broken
         if not self.is_suspended:
             if self.get_max_applications() == 0:
@@ -234,7 +231,7 @@ class ConditionalOffer(models.Model):
         if self.end_datetime:
             predicates.append(test_date > self.end_datetime)
         if any(predicates):
-            return 0
+            return False
         return self.get_max_applications(user) > 0
 
     def is_condition_satisfied(self, basket):
@@ -350,7 +347,10 @@ class ConditionalOffer(models.Model):
 
         def hide_time_if_zero(dt):
             # Only show hours/minutes if they have been specified
-            localtime = dt.astimezone(get_current_timezone())
+            if dt.tzinfo:
+                localtime = dt.astimezone(get_current_timezone())
+            else:
+                localtime = dt
             if localtime.hour == 0 and localtime.minute == 0:
                 return date_filter(localtime, settings.DATE_FORMAT)
             return date_filter(localtime, settings.DATETIME_FORMAT)
@@ -419,12 +419,12 @@ class Condition(models.Model):
     range = models.ForeignKey(
         'offer.Range', verbose_name=_("Range"), null=True, blank=True)
     type = models.CharField(_('Type'), max_length=128, choices=TYPE_CHOICES,
-                            null=True, blank=True)
-    value = PositiveDecimalField(_('Value'), decimal_places=2, max_digits=12,
-                                 null=True, blank=True)
+                            blank=True)
+    value = fields.PositiveDecimalField(
+        _('Value'), decimal_places=2, max_digits=12, null=True, blank=True)
 
-    proxy_class = models.CharField(_("Custom class"), null=True, blank=True,
-                                   max_length=255, unique=True, default=None)
+    proxy_class = fields.NullCharField(
+        _("Custom class"), max_length=255, unique=True, default=None)
 
     class Meta:
         verbose_name = _("Condition")
@@ -435,7 +435,7 @@ class Condition(models.Model):
         Return the proxy model
         """
         field_dict = dict(self.__dict__)
-        for field in field_dict.keys():
+        for field in list(field_dict.keys()):
             if field.startswith('_'):
                 del field_dict[field]
 
@@ -505,9 +505,10 @@ class Condition(models.Model):
             if not price:
                 continue
             line_tuples.append((price, line))
+        key = operator.itemgetter(0)
         if most_expensive_first:
-            return sorted(line_tuples, reverse=True)
-        return sorted(line_tuples)
+            return sorted(line_tuples, reverse=True, key=key)
+        return sorted(line_tuples, key=key)
 
 
 class Benefit(models.Model):
@@ -537,7 +538,7 @@ class Benefit(models.Model):
     # The value to use with the designated type.  This can be either an integer
     # (eg for multibuy) or a decimal (eg an amount) which is slightly
     # confusing.
-    value = PositiveDecimalField(
+    value = fields.PositiveDecimalField(
         _("Value"), decimal_places=2, max_digits=12, null=True, blank=True)
 
     # If this is not set, then there is no upper limit on how many products
@@ -549,8 +550,8 @@ class Benefit(models.Model):
 
     # A custom benefit class can be used instead.  This means the
     # type/value/max_affected_items fields should all be None.
-    proxy_class = models.CharField(_("Custom class"), null=True, blank=True,
-                                   max_length=255, unique=True, default=None)
+    proxy_class = fields.NullCharField(
+        _("Custom class"), max_length=255, unique=True, default=None)
 
     class Meta:
         verbose_name = _("Benefit")
@@ -558,7 +559,7 @@ class Benefit(models.Model):
 
     def proxy(self):
         field_dict = dict(self.__dict__)
-        for field in field_dict.keys():
+        for field in list(field_dict.keys()):
             if field.startswith('_'):
                 del field_dict[field]
 
@@ -725,7 +726,7 @@ class Benefit(models.Model):
             line_tuples.append((price, line))
 
         # We sort lines to be cheapest first to ensure consistent applications
-        return sorted(line_tuples)
+        return sorted(line_tuples, key=operator.itemgetter(0))
 
     def shipping_discount(self, charge):
         return D('0.00')
@@ -756,15 +757,14 @@ class Range(models.Model):
         verbose_name=_("Excluded Products"))
     classes = models.ManyToManyField(
         'catalogue.ProductClass', related_name='classes', blank=True,
-        verbose_name=_("Product Classes"))
+        verbose_name=_("Product Types"))
     included_categories = models.ManyToManyField(
         'catalogue.Category', related_name='includes', blank=True,
         verbose_name=_("Included Categories"))
 
     # Allow a custom range instance to be specified
-    proxy_class = models.CharField(
-        _("Custom class"), null=True, blank=True, max_length=255,
-        default=None, unique=True)
+    proxy_class = fields.NullCharField(
+        _("Custom class"), max_length=255, default=None, unique=True)
 
     date_created = models.DateTimeField(_("Date Created"), auto_now_add=True)
 
@@ -893,7 +893,7 @@ class Range(models.Model):
         """
         Test whether this product can be edited in the dashboard
         """
-        return self.proxy_class is None
+        return not self.proxy_class
 
 
 class RangeProduct(models.Model):
@@ -921,7 +921,7 @@ class CountCondition(Condition):
     def name(self):
         return self._description % {
             'count': self.value,
-            'range': unicode(self.range).lower()}
+            'range': six.text_type(self.range).lower()}
 
     @property
     def description(self):
@@ -1009,7 +1009,7 @@ class CoverageCondition(Condition):
     def name(self):
         return self._description % {
             'count': self.value,
-            'range': unicode(self.range).lower()}
+            'range': six.text_type(self.range).lower()}
 
     @property
     def description(self):
@@ -1112,7 +1112,7 @@ class ValueCondition(Condition):
     def name(self):
         return self._description % {
             'amount': currency(self.value),
-            'range': unicode(self.range).lower()}
+            'range': six.text_type(self.range).lower()}
 
     @property
     def description(self):
@@ -1205,7 +1205,7 @@ class ApplicationResult(object):
     # (a) Give a discount off the BASKET total
     # (b) Give a discount off the SHIPPING total
     # (a) Trigger a post-order action
-    BASKET, SHIPPING, POST_ORDER = range(0, 3)
+    BASKET, SHIPPING, POST_ORDER = list(range(0, 3))
     affects = None
 
     @property
