@@ -1,3 +1,4 @@
+from oscar.apps.address.abstract_models import AbstractPartnerAddress
 from oscar.apps.partner.abstract_models import (
     AbstractPartner, AbstractStockRecord, AbstractStockAlert)
 
@@ -6,6 +7,10 @@ class Partner(AbstractPartner):
     pass
 
 
+class PartnerAddress(AbstractPartnerAddress):
+    pass
+
+
 class StockRecord(AbstractStockRecord):
     pass
 
@@ -14,4 +19,4 @@ class StockAlert(AbstractStockAlert):
     pass
 
 
-from oscar.apps.partner.receivers import *
\ No newline at end of file
+from oscar.apps.partner.receivers import *  # noqa
