@@ -1,20 +1,24 @@
 from datetime import date
 from calendar import monthrange
 import re
+import six
 
 from django import forms
-from django.db.models import get_model
+from django.core.exceptions import ImproperlyConfigured
 from django.utils.translation import ugettext_lazy as _
 
+from oscar.core.loading import get_model
 from oscar.apps.address.forms import AbstractAddressForm
-from . import bankcards
-
 from oscar.views.generic import PhoneNumberMixin
+from . import bankcards
 
 Country = get_model('address', 'Country')
 BillingAddress = get_model('order', 'BillingAddress')
 Bankcard = get_model('payment', 'Bankcard')
 
+# List of card names for all the card types supported in payment.bankcards
+VALID_CARDS = set([card_type[0] for card_type in bankcards.CARD_TYPES])
+
 
 class BankcardNumberField(forms.CharField):
 
@@ -24,6 +28,13 @@ class BankcardNumberField(forms.CharField):
             'widget': forms.TextInput(attrs={'autocomplete': 'off'}),
             'label': _("Card number")
         }
+        if 'types' in kwargs:
+            self.accepted_cards = set(kwargs.pop('types'))
+            difference = self.accepted_cards - VALID_CARDS
+            if difference:
+                raise ImproperlyConfigured('The following accepted_cards are '
+                                           'unknown: %s' % difference)
+
         _kwargs.update(kwargs)
         super(BankcardNumberField, self).__init__(*args, **_kwargs)
 
@@ -38,6 +49,13 @@ class BankcardNumberField(forms.CharField):
         if value and not bankcards.luhn(value):
             raise forms.ValidationError(
                 _("Please enter a valid credit card number."))
+
+        if hasattr(self, 'accepted_cards'):
+            card_type = bankcards.bankcard_type(value)
+            if card_type not in self.accepted_cards:
+                raise forms.ValidationError(
+                    _("%s cards are not accepted." % card_type))
+
         return super(BankcardNumberField, self).clean(value)
 
 
@@ -106,11 +124,12 @@ class BankcardExpiryMonthField(BankcardMonthField):
         super(BankcardExpiryMonthField, self).__init__(*args, **_kwargs)
 
     def month_choices(self):
-        return [("%.2d" % x, "%.2d" % x) for x in xrange(1, 13)]
+        return [("%.2d" % x, "%.2d" % x) for x in six.moves.xrange(1, 13)]
 
     def year_choices(self):
-        return [(x, x) for x in xrange(date.today().year,
-                                       date.today().year + self.num_years)]
+        return [(x, x) for x in six.moves.xrange(
+            date.today().year,
+            date.today().year + self.num_years)]
 
     def clean(self, value):
         expiry_date = super(BankcardExpiryMonthField, self).clean(value)
@@ -146,14 +165,15 @@ class BankcardStartingMonthField(BankcardMonthField):
         super(BankcardStartingMonthField, self).__init__(*args, **_kwargs)
 
     def month_choices(self):
-        months = [("%.2d" % x, "%.2d" % x) for x in xrange(1, 13)]
+        months = [("%.2d" % x, "%.2d" % x) for x in six.moves.xrange(1, 13)]
         months.insert(0, ("", "--"))
         return months
 
     def year_choices(self):
         today = date.today()
-        years = [(x, x) for x in xrange(today.year - self.num_years,
-                                        today.year + 1)]
+        years = [(x, x) for x in six.moves.xrange(
+            today.year - self.num_years,
+            today.year + 1)]
         years.insert(0, ("", "--"))
         return years
 
@@ -200,6 +220,12 @@ class BankcardCCVField(forms.RegexField):
 
 
 class BankcardForm(forms.ModelForm):
+    # By default, this number field will accept any number. The only validation
+    # is whether it passes the luhn check. If you wish to only accept certain
+    # types of card, you can pass a types kwarg to BankcardNumberField, e.g.
+    #
+    # BankcardNumberField(types=[bankcards.VISA, bankcards.VISA_ELECTRON,])
+
     number = BankcardNumberField()
     ccv = BankcardCCVField()
     start_month = BankcardStartingMonthField()
