@@ -3,13 +3,14 @@ from decimal import Decimal as D
 from django.db import models
 from django.utils.translation import ugettext_lazy as _
 
-from oscar.core.utils import slugify
 from oscar.apps.shipping import Scales
+from oscar.models.fields import AutoSlugField
 
 
 class ShippingMethod(models.Model):
     # Fields from shipping.base.ShippingMethod must be added here manually.
-    code = models.SlugField(_("Slug"), max_length=128, unique=True)
+    code = AutoSlugField(_("Slug"), max_length=128, unique=True,
+                         populate_from='name')
     name = models.CharField(_("Name"), max_length=128, unique=True)
     description = models.TextField(_("Description"), blank=True)
 
@@ -18,6 +19,7 @@ class ShippingMethod(models.Model):
                                        blank=True, verbose_name=_("Countries"))
 
     is_discounted = False
+    discount = D('0.00')
     _basket = None
 
     class Meta:
@@ -25,11 +27,6 @@ class ShippingMethod(models.Model):
         verbose_name = _("Shipping Method")
         verbose_name_plural = _("Shipping Methods")
 
-    def save(self, *args, **kwargs):
-        if not self.code:
-            self.code = slugify(self.name)
-        super(ShippingMethod, self).save(*args, **kwargs)
-
     def __unicode__(self):
         return self.name
 
