@@ -1,5 +1,5 @@
 from django.contrib import admin
-from django.db.models import get_model
+from oscar.core.loading import get_model
 
 Voucher = get_model('voucher', 'Voucher')
 VoucherApplication = get_model('voucher', 'VoucherApplication')
@@ -11,7 +11,8 @@ class VoucherAdmin(admin.ModelAdmin):
     readonly_fields = ('num_basket_additions', 'num_orders', 'total_discount')
     fieldsets = (
         (None, {
-            'fields': ('name', 'code', 'usage', 'start_date', 'end_date')}),
+            'fields': ('name', 'code', 'usage', 'start_datetime',
+                       'end_datetime')}),
         ('Benefit', {
             'fields': ('offers',)}),
         ('Usage', {
