@@ -1,5 +1,6 @@
+from __future__ import print_function
 from django.core.management.base import BaseCommand, CommandError
-from django.db.models import get_model
+from oscar.core.loading import get_model
 
 from oscar.core.loading import get_class
 Order = get_model('order', 'Order')
@@ -26,5 +27,5 @@ class Command(BaseCommand):
         }
         messages = CommunicationEventType.objects.get_and_render(
             args[0], ctx)
-        print "Subject: %s\nBody:\n\n%s\nBody HTML:\n\n%s" % (
-            messages['subject'], messages['body'], messages['html'])
+        print("Subject: %s\nBody:\n\n%s\nBody HTML:\n\n%s" % (
+            messages['subject'], messages['body'], messages['html']))
