@@ -20,6 +20,15 @@ var oscar = (function(o, $) {
         }
     };
 
+    o.search = {
+        init: function() {
+            // Auto-submit (hidden) search form when selecting a new sort-by option
+            $('#id_sort_by').on('change', function() {
+                $(this).closest('form').submit();
+            });
+        }
+    };
+
     // This block may need removing after reworking of promotions app
     o.promotions = {
         init: function() {
@@ -51,8 +60,14 @@ var oscar = (function(o, $) {
             // prevent multiple submissions
             $('form[data-behaviours~="lock"]').submit(o.forms.submitIfNotLocked);
 
-            // Disable buttons when they are clicked
-            $('.js-disable-on-click').click(function(){$(this).button('loading');});
+            // Disable buttons when they are clicked and show a "loading" message taken from the
+            // data-loading-text attribute (http://getbootstrap.com/2.3.2/javascript.html#buttons).
+            // Do not disable if button is inside a form with invalid fields.
+            $('.js-disable-on-click').click(function(){
+                var form = $(this).parents("form");
+                if (!form || $(":invalid", form).length == 0)
+                    $(this).button('loading');
+            });
         },
         submitIfNotLocked: function(event) {
             var $form = $(this);
@@ -168,11 +183,15 @@ var oscar = (function(o, $) {
             $('#messages').html('');
             var payload = $('#basket_formset').serializeArray();
             $.post(o.basket.url, payload, o.basket.submitFormSuccess, 'json');
-            event.preventDefault();
+            if (event) {
+                event.preventDefault();
+            }
         },
         submitFormSuccess: function(data) {
             $('#content_inner').html(data.content_html);
-            $('#messages').html('');
+
+            // Show any flash messages
+            o.messages.clear();
             for (var level in data.messages) {
                 for (var i=0; i<data.messages[level].length; i++) {
                     o.messages[level](data.messages[level][i]);
