@@ -1,5 +1,5 @@
 from django import template
-from django.db.models import get_model
+from oscar.core.loading import get_model
 
 from oscar.core.loading import get_class
 
