@@ -1,7 +1,5 @@
-import httplib
+from six.moves import http_client
 
-from django.test import TestCase
-from django.test.client import Client
 from django.core.urlresolvers import reverse
 from django.contrib.auth.models import Permission
 from django_webtest import WebTest
@@ -15,90 +13,49 @@ User = get_user_model()
 
 def add_permissions(user, permissions):
     """
+    Grant permissions to the passed user
+
     :param permissions: e.g. ['partner.dashboard_access']
     """
     for permission in permissions:
-        app_label, _, codename = permission.partition('.')
+        app_label, __, codename = permission.partition('.')
         perm = Permission.objects.get(content_type__app_label=app_label,
                                       codename=codename)
         user.user_permissions.add(perm)
 
 
-class ClientTestCase(TestCase):
-    """
-    Helper TestCase for using Django's test client.  The class provides
-    auto-creation of a user to avoid boilerplate code.
-    """
-    username = 'dummyuser'
-    email = 'dummyuser@example.com'
-    password = 'staffpassword'
-    is_anonymous = False
-    is_staff = False
-    is_superuser = False
-    permissions = []
-
-    def setUp(self):
-        self.client = Client()
-        if not self.is_anonymous:
-            self.login()
-
-    def login(self):
-        self.user = self.create_user()
-        self.client.login(email=self.email,
-                          password=self.password)
-
-    def create_user(self, username=None, password=None, email=None,
-                    is_staff=None, is_superuser=None, permissions=None):
-        user = User.objects.create_user(username or self.username,
-                                        email or self.email,
-                                        password or self.password)
-        user.is_staff = is_staff or self.is_staff
-        user.is_superuser = is_superuser or self.is_superuser
-        user.save()
-
-        perms = permissions if permissions is not None else self.permissions
-        add_permissions(user, perms)
-        return user
-
-    def assertIsRedirect(self, response, expected_url=None):
-        self.assertTrue(response.status_code in (httplib.FOUND,
-                                                 httplib.MOVED_PERMANENTLY))
-        if expected_url:
-            location = URL.from_string(response['Location'])
-            self.assertEqual(expected_url, location.path())
-
-    def assertRedirectUrlName(self, response, name, kwargs=None):
-        self.assertIsRedirect(response)
-        location = response['Location'].replace('http://testserver', '')
-        self.assertEqual(location, reverse(name, kwargs=kwargs))
-
-    def assertIsOk(self, response):
-        self.assertEqual(httplib.OK, response.status_code)
-
-    def assertNoAccess(self, response):
-        self.assertTrue(response.status_code in (httplib.NOT_FOUND,
-                                                 httplib.FORBIDDEN))
-
-    def assertInContext(self, response, key):
-        self.assertTrue(key in response.context,
-                        "Context should contain a variable '%s'" % key)
-
-
 class WebTestCase(WebTest):
     is_staff = False
-    is_anonymous = True
+    is_anonymous = False
+    is_superuser = False
+
     username = 'testuser'
     email = 'testuser@buymore.com'
     password = 'somefancypassword'
+    permissions = []
 
     def setUp(self):
         self.user = None
-        if not self.is_anonymous or self.is_staff:
-            self.user = User.objects.create_user(self.username, self.email,
-                                                 self.password)
+
+        if not self.is_anonymous:
+            self.user = self.create_user(
+                self.username, self.email, self.password)
             self.user.is_staff = self.is_staff
+            add_permissions(self.user, self.permissions)
             self.user.save()
 
+    def create_user(self, username=None, email=None, password=None):
+        """
+        Create a user for use in a test.
+
+        As usernames are optional in newer versions of Django, it only sets it
+        if exists.
+        """
+        kwargs = {'email': email, 'password': password}
+        if 'username' in User._meta.get_all_field_names():
+            kwargs['username'] = username
+        return User.objects.create_user(**kwargs)
+
     def get(self, url, **kwargs):
         kwargs.setdefault('user', self.user)
         return self.app.get(url, **kwargs)
@@ -110,8 +67,8 @@ class WebTestCase(WebTest):
     # Custom assertions
 
     def assertIsRedirect(self, response, expected_url=None):
-        self.assertTrue(response.status_code in (httplib.FOUND,
-                                                 httplib.MOVED_PERMANENTLY))
+        self.assertTrue(response.status_code in (
+            http_client.FOUND, http_client.MOVED_PERMANENTLY))
         if expected_url:
             location = URL.from_string(response['Location'])
             self.assertEqual(expected_url, location.path())
@@ -121,3 +78,25 @@ class WebTestCase(WebTest):
         location = response.headers['Location']
         redirect_path = location.replace('http://localhost:80', '')
         self.assertEqual(reverse(url_name), redirect_path)
+
+    def assertNoAccess(self, response):
+        self.assertContext(response)
+        self.assertTrue(response.status_code in (http_client.NOT_FOUND,
+                                                 http_client.FORBIDDEN))
+
+    def assertRedirectUrlName(self, response, name, kwargs=None):
+        self.assertIsRedirect(response)
+        location = response['Location'].replace('http://testserver', '')
+        self.assertEqual(location, reverse(name, kwargs=kwargs))
+
+    def assertIsOk(self, response):
+        self.assertEqual(http_client.OK, response.status_code)
+
+    def assertContext(self, response):
+        self.assertTrue(response.context is not None,
+                        'No context was returned')
+
+    def assertInContext(self, response, key):
+        self.assertContext(response)
+        self.assertTrue(key in response.context,
+                        "Context should contain a variable '%s'" % key)
